import { BaseError, } from 'viem';
import { getChainId } from 'viem/actions';
import { abstractTestnet } from 'viem/chains';
import { assertCurrentChain, getAction, parseAccount } from 'viem/utils';
import {} from 'viem/zksync';
import { assertEip712Request, } from '../eip712.js';
import { AccountNotFoundError } from '../errors/account.js';
const ALLOWED_CHAINS = [abstractTestnet.id];
function convertCallsToPermissionlessCalls(calls) {
    return (calls || []).map((call) => ({
        to: call.target,
        value: call.value.toString(),
        data: call.callData,
    }));
}
function convertBigIntToString(obj) {
    if (typeof obj === 'bigint') {
        return obj.toString();
    }
    if (typeof obj !== 'object' || obj === null) {
        return obj;
    }
    if (Array.isArray(obj)) {
        return obj.map(convertBigIntToString);
    }
    return Object.fromEntries(Object.entries(obj).map(([key, value]) => [
        key,
        convertBigIntToString(value),
    ]));
}
export async function sendPrivyTransaction(client, signerClient, args, useSignerAddress = false) {
    const { account: account_ = client.account, chain = client.chain, ...transaction } = args;
    // TODO: open up typing to allow for eip712 transactions
    transaction.type = 'eip712';
    transaction.value = transaction.value
        ? BigInt(transaction.value)
        : 0n;
    if (!account_)
        throw new AccountNotFoundError({
            docsPath: '/docs/actions/wallet/signTransaction',
        });
    const smartAccount = parseAccount(account_);
    const fromAccount = useSignerAddress ? signerClient.account : smartAccount;
    assertEip712Request({
        account: smartAccount,
        chain,
        ...transaction,
    });
    if (!chain || !ALLOWED_CHAINS.includes(chain.id)) {
        throw new BaseError('Invalid chain specified');
    }
    if (!chain?.custom?.getEip712Domain)
        throw new BaseError('`getEip712Domain` not found on chain.');
    if (!chain?.serializers?.transaction)
        throw new BaseError('transaction serializer not found on chain.');
    const chainId = await getAction(client, getChainId, 'getChainId')({});
    if (chain !== null)
        assertCurrentChain({
            currentChainId: chainId,
            chain: chain,
        });
    const calls = [
        {
            target: transaction.to,
            allowFailure: false,
            value: BigInt(transaction.value ?? 0),
            callData: transaction.data ?? '0x',
        },
    ];
    const serializedTransaction = convertBigIntToString(transaction);
    const result = (await client.request({
        method: 'privy_sendSmartWalletTx',
        params: [
            fromAccount,
            serializedTransaction,
            convertCallsToPermissionlessCalls(calls),
        ],
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
    }, { retryCount: 0 }));
    return result;
}
//# sourceMappingURL=sendPrivyTransaction.js.map